import tempfile

from django.core.files import File
from wagtail.wagtailimages.models import get_image_model
import requests

ImgModel = get_image_model()


class ImageNotAvailable(Exception):
    pass


def retrieve_file_from_url(url_to_image):
    """
    Returns a 2-tuple of a filename and a django File object instance of
    an image at `url_to_image`

    Raises ImageNotAvailable if an image could not be retrieved.
    """
    # Stream the image from the url
    request = requests.get(url_to_image, stream=True)
    # Was the request OK?
    if request.status_code == 200:
        # Get the filename from the url, used for saving later
        filename = url_to_image.split('/')[-1]
        # Create a temporary file
        streamed_file = tempfile.NamedTemporaryFile()
        # Read the streamed image in sections
        for block in request.iter_content(1024 * 8):

            # If no more file then stop
            if not block:
                break

            # Write image block to temporary file
            streamed_file.write(block)
        temp_file = File(streamed_file)
        temp_file.name = filename
        return temp_file
    else:
        raise ImageNotAvailable(
            'There was a problem retrieving an image from: {}'.format(
                url_to_image
            )
        )


def create_wagtail_image_from_url(image_url, image_title, image_filename=None):
    image_file = retrieve_file_from_url(image_url)
    if image_filename:
        image_file.name = image_filename
    image = ImgModel(
        title=image_title,
        file=image_file
    )
    image.save()
    return image



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError

import coveapi

from wagtail_stacks_embed.registry import stacks_embed_service_registry
from wagtail_stacks_embed.utils import create_wagtail_image_from_url

COVE_API_ID = getattr(settings, 'COVE_API_ID', None)
COVE_API_SECRET = getattr(settings, 'COVE_API_SECRET', None)

if not COVE_API_ID or not COVE_API_SECRET:
    raise ImproperlyConfigured(
        "In order to use the PBS COVE service with you must "
        "define both COVE_API_ID and COVE_API_SECRET in your settings file."
    )

cove_api = coveapi.connect(COVE_API_ID, COVE_API_SECRET)

COVE_EMBED_CODE_TEMPLATE = (
    "<iframe  id='partnerPlayer' frameborder='0' marginwidth='0' "
    "marginheight='0' scrolling='no' width='__WIDTH__' height='__HEIGHT__' "
    "src='http://video.pbs.org/partnerplayer/__EMBEDKEY__/"
    "?autoplay=true&start=0&end=0&chapterbar=true&topbar="
    "false&endscreen=true' allowfullscreen></iframe>"
)


def process_cove_video(instance):
    """
    Processes a StacksEmbed instance associated with the COVE service

    Expects that instance.canonical_url is to a valid video on video.pbs.org
    """
    instance.id_on_service = instance.canonical_url.split('/')[-2]
    cove_video = cove_api.videos.filter(
        filter_tp_media_object_id=instance.id_on_service,
        fields='associated_images'
    )
    if cove_video.get('count') == 1:
        results = cove_video.get('results')[0]
        instance.name = results.get('title')
        embed_code_token = results.get('partner_player').strip().split(
            '?')[0].rstrip('/').split('/')[-1]
        embed_code = COVE_EMBED_CODE_TEMPLATE.replace(
            "__EMBEDKEY__",
            embed_code_token
        )
        instance.embed_code = embed_code
        instance.processed = True
        if not instance.poster_image:
            image_url = None
            for d in results.get('associated_images'):
                if d['type']['eeid'].lower() == 'mezzanine':
                    image_url = d['url']
                    break
            if image_url is not None:
                img = create_wagtail_image_from_url(image_url, instance.name)
                instance.poster_image = img
    else:
        raise ValidationError(
            '{} is not a URL to a valid COVE video. '
            'Please try again!'.format(instance.canonical_url)
        )

stacks_embed_service_registry.add_embed_service(
    short_name='pbs_cove',
    verbose_name='COVE (PBS Video)',
    allowed_domains=['video.pbs.org'],
    process_instance_func=process_cove_video
)



from django.core.exceptions import ValidationError

import requests
from bs4 import BeautifulSoup

from wagtail_stacks_embed.registry import stacks_embed_service_registry
from wagtail_stacks_embed.utils import create_wagtail_image_from_url


def process_vimeo_url(instance):
    """
    Processes a StacksEmbed instance associated with Vimeo

    Expects that instance.canonical_url is to a valid video on either
    www.youtube.com or youtu.be
    """
    vimeo_page = requests.get(instance.canonical_url)

    if vimeo_page.status_code == 200:
        vimeo_page_parsed = BeautifulSoup(vimeo_page.content, 'html5lib')
        instance.canonical_url = vimeo_page_parsed.find(
            property="og:url"
        )['content']
        instance.id_on_service = instance.canonical_url.split('/')[-1]
        instance.name = vimeo_page_parsed.find(property="og:title")['content']
        image_url = vimeo_page_parsed.find(
            property="og:image"
        )['content'].replace('https', 'http')
        if image_url and not instance.poster_image:
            filename = image_url.split('/')[-1]
            filename = '{}-{}'.format(instance.id_on_service, filename)
            img = create_wagtail_image_from_url(
                image_url,
                instance.name,
                image_filename=filename
            )
            instance.poster_image = img
        instance.embed_code = (
            '<iframe src="//player.vimeo.com/video/{}" width="500" '
            'height="281" frameborder="0" webkitallowfullscreen '
            'mozallowfullscreen allowfullscreen></iframe>'
        ).format(instance.id_on_service)
    else:
        if vimeo_page.status_code == 404:
            error_msg = (
                '{} is an unavailable URL on Vimeo. '
                'Double-check it and try again!'
            )
        else:
            error_msg = (
                'There was an error processing {}. Please try again later!'
            )
        raise ValidationError(error_msg.format(instance.canonical_url))

stacks_embed_service_registry.add_embed_service(
    short_name='vimeo',
    verbose_name='Vimeo',
    allowed_domains=['vimeo.com'],
    process_instance_func=process_vimeo_url
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_stacks_embed', '0002_auto_20151002_1353'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='stacksembed',
            options={'ordering': ('-date_created', '-date_modified'), 'verbose_name': 'Embeddable Media', 'verbose_name_plural': 'Embeddable Media'},
        ),
        migrations.AlterField(
            model_name='stacksembed',
            name='canonical_url',
            field=models.URLField(help_text='Allowed domains: soundcloud.com www.youtube.com video.pbs.org vimeo.com youtu.be', verbose_name='Embed URL'),
        ),
        migrations.AlterField(
            model_name='stacksembed',
            name='service',
            field=models.CharField(help_text=b'The service you want to embed from.', max_length=30, verbose_name='Service', choices=[(b'pbs_cove', b'COVE (PBS Video)'), (b'youtube', b'YouTube'), (b'soundcloud', b'SoundCloud'), (b'vimeo', b'Vimeo')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def pbs_cove_allowfullscreen(apps, schema_editor):
    """
    Updates all existing all PBS COVE partner play embed codes to include
    'allowfullscreen'.
    """
    StacksEmbed = apps.get_model("wagtail_stacks_embed", "StacksEmbed")
    for embed in StacksEmbed.objects.filter(service='pbs_cove'):
        embed_code = embed.embed_code.replace(
            "'></iframe>",
            "' allowfullscreen></iframe>"
        )
        embed.embed_code = embed_code
        embed.save()


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_stacks_embed', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(pbs_cove_allowfullscreen)
    ]



import copy

from django.conf import settings


class AlreadyRegistered(Exception):
    pass


class NotRegistered(Exception):
    pass


class AlreadyRegisteredDomain(Exception):
    pass


class StacksEmbedServiceRegistry(object):
    """
    A StacksEmbedServiceRegistry object allows Embed Services to be
    registered on an app-by-app basis.
    """

    def __init__(self):
        self._registry = {}
        self._allowed_domains = {}

    def add_embed_service(self, short_name, verbose_name,
                          allowed_domains, process_instance_func):
        """
        Registers an embed service.
        * short_name: What will be stored in the database for
                      StacksEmbed.service
        * verbose_name: The human-readable name of the service (what will be
                        shown to users for StacksEmbed.service)
        * process_instance_func: A function that accepts an instance of
                                 StacksEmbed and does any necessary pre_save
                                 processing to properly build an embed code
                                 and assign it to StacksEmbed.embed_code
        """
        if short_name in self._registry:
            raise AlreadyRegistered(
                'A service with the short_name {} is already registered with '
                'the Stacks Embed Service Registry.'.format(short_name)
            )
        else:
            for domain in allowed_domains:
                if domain not in self._allowed_domains:
                    self._allowed_domains[domain] = short_name
                else:
                    raise AlreadyRegisteredDomain(
                        "'{}' has been already registered with "
                        "the '{}' service.".format(
                            domain,
                            self._registry[self._allowed_domains[domain]][0]
                        )
                    )
            self._registry[short_name] = (
                verbose_name, process_instance_func
            )

    def remove_embed_service(self, short_name):
        """
        Unregisters an embed service.

        If an embed service isn't already registered to `short_name`
        NotRegistered will raise.
        """
        if short_name not in self._registry:
            raise NotRegistered(
                'No Embed Service is registered to {}' % short_name
            )
        else:
            del self._registry[short_name]

stacks_embed_service_registry = StacksEmbedServiceRegistry()


def find_stacks_embed_services():
    """
    Auto-discover INSTALLED_APPS stacks_embed_service.py modules and fail
    silently when not present. This forces an import on them (thereby
    registering them)

    This is a near 1-to-1 copy of how django's admin application registers
    models.
    """
    from importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's sizedimage module.
        try:
            before_import_registry = copy.copy(
                stacks_embed_service_registry._registry
            )
            import_module('{}.stacks_embed_service'.format(app))
        except:
            # Reset the stacks_embed_service_registry to the state before the
            # last import as this import will have to reoccur on the next
            # request and this could raise NotRegistered and AlreadyRegistered
            # exceptions (see django ticket #8245).
            stacks_embed_service_registry.\
                _modelstacks_embed_service_registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have a stuff module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'stacks_embed_service'):
                raise


find_stacks_embed_services()



from copy import deepcopy

from wagtail.wagtailcore.blocks import (
    TextBlock,
    RichTextBlock
)
from wagtail.wagtailsnippets.blocks import SnippetChooserBlock

from streamfield_tools.blocks import (
    Rendition,
    RenditionAwareListBlock,
    MultiRenditionStructBlock,
    RenditionAwareStructBlock
)

from .models import StacksEmbed

stacks_embed_form_blocks = [
    ('embed', SnippetChooserBlock(StacksEmbed)),
    ('overline', TextBlock(required=False)),
    ('title', TextBlock(required=False)),
    ('additional_content', RichTextBlock(required=False))
]

stacks_embed_block = MultiRenditionStructBlock(
    deepcopy(stacks_embed_form_blocks),
    core_renditions=(
        Rendition(
            short_name='full_width',
            verbose_name='Full Width Embed',
            description="An embed that spans the full width of it's "
                        "containing div.",
            path_to_template='wagtail_stacks_embed/single/full_width.html',
            image_rendition='fill-1600x901'
        ),
        Rendition(
            short_name='embed_left_content_right',
            verbose_name='Embed Left, Content Right',
            description="Display an embed on the left with it's "
                        "accompanying content on the right.",
            path_to_template='wagtail_stacks_embed/single/'
                             'embed_left_content_right.html',
            image_rendition='fill-800x450'
        ),
        Rendition(
            short_name='embed_right_content_left',
            verbose_name='Embed Right, Content Left',
            description="Display an embed on the right with it's accompanying "
                        "content on the left.",
            path_to_template='wagtail_stacks_embed/single/'
                             'embed_right_content_left.html',
            image_rendition='fill-800x450'
        )
    ),
    addl_renditions_settings_key='stacks_embed_block',
    label='Media Embed',
    icon='snippet'
)


stacks_embedlist_block = MultiRenditionStructBlock(
    [
        ('title', TextBlock(required=False)),
        ('embeds', RenditionAwareListBlock(
            RenditionAwareStructBlock(
                deepcopy(stacks_embed_form_blocks),
                template='wagtail_stacks_embed/list/list-item.html',
                template_carousel='wagtail_stacks_embed/list/'
                                  'list-item-carousel.html'
            )
        ))
    ],
    core_renditions=(
        Rendition(
            short_name='1up',
            verbose_name="Embed List 1-Up",
            description="A list of embeds displayed in grid with one image "
                        "in each row.",
            path_to_template='wagtail_stacks_embed/list/1up.html',
            image_rendition='fill-1600x901'
        ),
        Rendition(
            short_name='2up',
            verbose_name="Embed List 2-Up",
            description="A list of embeds displayed in grid with two "
                        "in each row.",
            path_to_template='wagtail_stacks_embed/list/2up.html',
            image_rendition='fill-800x450'
        ),
        Rendition(
            short_name='3up',
            verbose_name="Embed List 3-Up",
            description="A list of embeds displayed in grid with three "
                        "in each row.",
            path_to_template='wagtail_stacks_embed/list/3up.html',
            image_rendition='fill-700x390'
        ),
        Rendition(
            short_name='carousel',
            verbose_name="Embed Carousel / Gallery",
            description="A list of embeds displayed in a javascript-powered "
                        "carousel.",
            path_to_template='wagtail_stacks_embed/list/carousel.html',
            image_rendition='fill-1600x901'
        )
    ),
    addl_renditions_settings_key='stacks_embedlist_block',
    label='Media Embed List',
    icon='snippet'
)



from django.core.exceptions import ValidationError

import requests
from bs4 import BeautifulSoup

from wagtail_stacks_embed.registry import stacks_embed_service_registry
from wagtail_stacks_embed.utils import create_wagtail_image_from_url


def process_youtube_video(instance):
    """
    Processes a StacksEmbed instance associated with the YouTube service

    Expects that instance.canonical_url is to a valid video on either
    www.youtube.com or youtu.be
    """
    youtube_page = requests.get(instance.canonical_url)

    if youtube_page.status_code == 200:
        youtube_parsed = BeautifulSoup(youtube_page.content, 'html5lib')
        try:
            official_url = youtube_parsed.find(property="og:url")['content']
        except TypeError:
            raise ValidationError(
                '{} is an unavailable URL on YouTube. '
                'Double-check it and try again!'.format(instance.canonical_url)
            )
        else:
            instance.id_on_service = official_url.split('=')[-1]
            instance.name = youtube_parsed.find(property="og:title")['content']
            image_url = youtube_parsed.find(property="og:image")['content']
            if image_url and not instance.poster_image:
                filename = image_url.split('/')[-1]
                filename = '{}-{}'.format(instance.id_on_service, filename)
                img = create_wagtail_image_from_url(
                    image_url,
                    instance.name,
                    image_filename=filename
                )
                instance.poster_image = img
            instance.embed_code = (
                '<iframe width="__WIDTH__" height="__HEIGHT__" '
                'src="https://www.youtube.com/'
                'embed/{}" frameborder="0" allowfullscreen></iframe>'
            ).format(instance.id_on_service)

stacks_embed_service_registry.add_embed_service(
    short_name='youtube',
    verbose_name='YouTube',
    allowed_domains=[
        'www.youtube.com',
        'youtu.be'
    ],
    process_instance_func=process_youtube_video
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailimages', '0006_add_verbose_names'),
    ]

    operations = [
        migrations.CreateModel(
            name='StacksEmbed',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(help_text='The internal name/signifier of this content.', max_length=100, verbose_name='Name', blank=True)),
                ('service', models.CharField(help_text=b'The service you want to embed from.', max_length=30, verbose_name='Service', choices=[(b'pbs_cove', b'COVE (PBS Video)')])),
                ('id_on_service', models.CharField(help_text=b'The ID of the content on the service.', max_length=120, verbose_name='Service ID')),
                ('embed_code', models.TextField(verbose_name='Embed Code')),
                ('canonical_url', models.URLField(help_text='Allowed domains: - video.pbs.org', verbose_name='Embed URL')),
                ('process_on_save', models.BooleanField(default=True, help_text=b'A boolean (True/False) signifying if this instance should be processed by its associated API on save.', verbose_name='Process on Save?')),
                ('poster_image', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='wagtailimages.Image', help_text=b'NOTE: An image will be automatically generated from the URL.', null=True, verbose_name='Poster Image')),
            ],
            options={
                'verbose_name': 'Embeddable Media',
                'verbose_name_plural': 'Embeddable Media',
            },
        ),
    ]



from urlparse import urlparse

from django.core.exceptions import ValidationError
from django.db import models
from django.dispatch import receiver
from django.utils import six
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailadmin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.wagtailimages.edit_handlers import ImageChooserPanel
from wagtail.wagtailimages.models import get_image_model
from wagtail.wagtailsearch import index
from wagtail.wagtailsnippets.models import register_snippet

from .registry import stacks_embed_service_registry

STACKS_EMBED_CHOICES = [
    (key, value[0])
    for key, value in six.iteritems(
        stacks_embed_service_registry._registry
    )
]

models.options.DEFAULT_NAMES = models.options.DEFAULT_NAMES + ('description',)

supported_domains = ', '.join(
    [
        '{}'.format(key)
        for key, value in six.iteritems(
            stacks_embed_service_registry._allowed_domains
        )
    ]
)


@python_2_unicode_compatible
@register_snippet
class StacksEmbed(models.Model, index.Indexed):
    """
    Represents an instance of embeddable media
    """
    date_created = models.DateTimeField(
        auto_now_add=True
    )
    date_modified = models.DateTimeField(
        auto_now=True
    )
    name = models.CharField(
        _('Name'),
        max_length=100,
        help_text=_('The internal name/signifier of this content.'),
        blank=True
    )
    poster_image = models.ForeignKey(
        get_image_model(),
        verbose_name=_('Poster Image'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=(
            'NOTE: An image will be automatically generated from the URL.'
        )
    )
    service = models.CharField(
        _('Service'),
        choices=STACKS_EMBED_CHOICES,
        max_length=30,
        help_text="The service you want to embed from."
    )
    id_on_service = models.CharField(
        _('Service ID'),
        max_length=120,
        help_text="The ID of the content on the service."
    )
    embed_code = models.TextField(
        _('Embed Code')
    )
    canonical_url = models.URLField(
        _('Embed URL'),
        help_text=_(
            'Allowed domains: {}'.format(
                ' '.join([
                    '{}'.format(key)
                    for key, value in six.iteritems(
                        stacks_embed_service_registry._allowed_domains
                    )
                ])
            )
        )
    )
    process_on_save = models.BooleanField(
        _('Process on Save?'),
        default=True,
        help_text=(
            'A boolean (True/False) signifying if this instance should be '
            'processed by its associated API on save.'
        )
    )
    panels = [
        FieldPanel('canonical_url'),
        MultiFieldPanel(
            [
                FieldPanel('name'),
                ImageChooserPanel('poster_image'),
                FieldPanel('process_on_save'),
            ],
            heading="Metadata Fields",
            classname="collapsible collapsed"
        )
    ]
    search_fields = [
        index.SearchField('name', partial_match=True),
        index.SearchField('service', partial_match=True),
    ]

    def __str__(self):
        return u"{} | {}".format(self.name, self.get_service_display())

    class Meta:
        verbose_name = _('Embeddable Media')
        verbose_name_plural = _('Embeddable Media')
        ordering = ('-date_created', '-date_modified')
        description = 'Supported domains: {}'.format(supported_domains)

    def clean_canonical_url(self):
        """
        Ensures `canonical_url` is associated with a domain registered with
        the Stacks Embed Service Registry
        """
        data = self.cleaned_data['canonical_url']
        canonical_url_parsed = urlparse(data)
        if (
            canonical_url_parsed.netloc
        ) not in stacks_embed_service_registry._allowed_domains:
            raise ValidationError(
                "{} is not an allowed. Only URLS from the following "
                "domains can be used: {}".format(
                    data,
                    ", ".join([
                        key
                        for key, value in six.iteritems(
                            stacks_embed_service_registry._allowed_domains
                        )
                    ])
                )
            )
        return data

    def clean(self):
        canonical_url_parsed = urlparse(self.canonical_url)
        if (
            canonical_url_parsed.netloc
        ) not in stacks_embed_service_registry._allowed_domains:
            raise ValidationError(
                "{} is a URL from an unallowed domain. Only URLS from the "
                "following domains can be used: {}".format(
                    self.canonical_url,
                    ", ".join([
                        key
                        for key, value in six.iteritems(
                            stacks_embed_service_registry._allowed_domains
                        )
                    ])
                )
            )
        else:
            self.service = stacks_embed_service_registry.\
                _allowed_domains.get(canonical_url_parsed.netloc)
            if self.process_on_save is True:
                stacks_embed_service_registry._registry[
                    self.service
                ][1](self)
                self.process_on_save = False


@receiver(models.signals.pre_save, sender=StacksEmbed)
def process_service(sender, instance, **kwargs):
    """
    Ensures StacksEmbed.full_clean() is ALWAYS called.
    """
    try:
        instance.full_clean()
    except ValidationError as e:
        # Do something based on the errors contained in e.message_dict.
        # Display them to a user, or handle them programmatically.
        pass
        del e



from streamfield_tools.registry import block_registry

from .blocks import (
    stacks_embed_block,
    stacks_embedlist_block
)

block_registry.register_block(
    'embed',
    stacks_embed_block
)

block_registry.register_block(
    'embed_list',
    stacks_embedlist_block
)



from django.core.exceptions import ValidationError

import requests
from bs4 import BeautifulSoup

from wagtail_stacks_embed.registry import stacks_embed_service_registry
from wagtail_stacks_embed.utils import create_wagtail_image_from_url


def process_soundcloud_url(instance):
    """
    Processes a StacksEmbed instance associated with the YouTube service

    Expects that instance.canonical_url is to a valid video on either
    www.youtube.com or youtu.be
    """
    sc_page = requests.get(instance.canonical_url)

    if sc_page.status_code == 200:
        sc_page_parsed = BeautifulSoup(sc_page.content)
        instance.id_on_service = sc_page_parsed.find(
            property="al:ios:url"
        )['content'].split(':')[-1]
        instance.name = sc_page_parsed.find(property="og:title")['content']
        image_url = sc_page_parsed.find(
            property="twitter:image:src"
        )['content'].replace('https', 'http')
        if image_url is not None:
            img = create_wagtail_image_from_url(image_url, instance.name)
            instance.poster_image = img
        instance.embed_code = (
            '<iframe width="100%" height="450" scrolling="no" '
            'frameborder="no" src="https://w.soundcloud.com/player/?url='
            'https%3A//api.soundcloud.com/tracks/{}&amp;auto_play=false&amp;'
            'hide_related=false&amp;show_comments=true&amp;show_user=true&amp;'
            'show_reposts=false&amp;visual=true"></iframe>'
        ).format(instance.id_on_service)
    else:
        if sc_page.status_code == 404:
            error_msg = (
                '{} is an unavailable URL on SoundCloud. '
                'Double-check it and try again!'
            )
        else:
            error_msg = (
                'There was an error processing {}. Please try again later!'
            )
        raise ValidationError(error_msg.format(instance.canonical_url))

stacks_embed_service_registry.add_embed_service(
    short_name='soundcloud',
    verbose_name='SoundCloud',
    allowed_domains=['soundcloud.com'],
    process_instance_func=process_soundcloud_url
)
